@@ -3,8 +3,8 @@
 **  LICENSE file, found in the top level directory of this distribution. If you
 **  did not receive the LICENSE file with this file, you may obtain it from the
 **  Vidalia source package distributed by the Vidalia Project at
-**  http://www.torproject.org/projects/vidalia.html. No part of Vidalia, 
-**  including this file, may be copied, modified, propagated, or distributed 
+**  http://www.torproject.org/projects/vidalia.html. No part of Vidalia,
+**  including this file, may be copied, modified, propagated, or distributed
 **  except according to the terms described in the LICENSE file.
 */
 
@@ -105,6 +105,8 @@ MainWindow::MainWindow()
 
   ui.setupUi(this);
 
+  _warnTimer = new QTimer();
+
   /* Pressing 'Esc' or 'Ctrl+W' will close the window */
   Vidalia::createShortcut("Ctrl+W", this, ui.btnHide, SLOT(click()));
   Vidalia::createShortcut("Esc", this, ui.btnHide, SLOT(click()));
@@ -135,7 +137,7 @@ MainWindow::MainWindow()
   updateTorStatus(Stopped);
 
   /* Create a new TorControl object, used to communicate with Tor */
-  _torControl = Vidalia::torControl(); 
+  _torControl = Vidalia::torControl();
   connect(_torControl, SIGNAL(started()), this, SLOT(started()));
   connect(_torControl, SIGNAL(startFailed(QString)),
           this, SLOT(startFailed(QString)));
@@ -143,7 +145,7 @@ MainWindow::MainWindow()
           this, SLOT(stopped(int, QProcess::ExitStatus)));
   connect(_torControl, SIGNAL(connected()), this, SLOT(connected()));
   connect(_torControl, SIGNAL(disconnected()), this, SLOT(disconnected()));
-  connect(_torControl, SIGNAL(connectFailed(QString)), 
+  connect(_torControl, SIGNAL(connectFailed(QString)),
           this, SLOT(connectFailed(QString)));
   connect(_torControl, SIGNAL(authenticated()), this, SLOT(authenticated()));
   connect(_torControl, SIGNAL(authenticationFailed(QString)),
@@ -162,6 +164,8 @@ MainWindow::MainWindow()
           this, SLOT(circuitEstablished()));
   connect(_torControl, SIGNAL(dangerousPort(quint16, bool)),
           this, SLOT(warnDangerousPort(quint16, bool)));
+  connect(_torControl, SIGNAL(logMessage(tc::Severity, QString)),
+          this, SLOT(log(tc::Severity, QString)));
 
   /* Create a new HelperProcess object, used to start the web browser */
   _browserProcess = new HelperProcess(this);
@@ -186,6 +190,8 @@ MainWindow::MainWindow()
   connect(vApp, SIGNAL(running()), this, SLOT(running()));
   connect(vApp, SIGNAL(aboutToQuit()), this, SLOT(aboutToQuit()));
 
+  connect(_warnTimer, SIGNAL(timeout()), this, SLOT(warnButton()));
+
 #if defined(USE_AUTOUPDATE)
   /* Create a timer used to remind us to check for software updates */
   connect(&_updateTimer, SIGNAL(timeout()), this, SLOT(checkForUpdates()));
@@ -216,7 +222,7 @@ MainWindow::MainWindow()
 
   ui.chkShowOnStartup->setChecked(settings.showMainWindowAtStart());
   if (ui.chkShowOnStartup->isChecked())
-    show(); 
+    show();
   /* Optimistically hope that the tray icon gets added. */
   _trayIcon.show();
 
@@ -227,8 +233,10 @@ MainWindow::MainWindow()
       TransformProcessType(&psn, kProcessTransformToForegroundApplication);
   }
   /* Vidalia launched in background (LSUIElement=true). Bring to foreground. */
-  VidaliaWindow::setVisible(true);
+  VidaliaWindow::setVisible(settings.showMainWindowAtStart());
 #endif
+
+  _flashToggle = false;
 }
 
 /** Destructor. */
@@ -357,7 +365,7 @@ MainWindow::aboutToQuit()
   vNotice("Cleaning up before exiting.");
 
   if (_torControl->isVidaliaRunningTor()) {
-    /* Kill our Tor process now */ 
+    /* Kill our Tor process now */
     _torControl->stop();
   }
 
@@ -389,7 +397,7 @@ MainWindow::aboutToQuit()
 #endif
 
     if (_imProcess->state() == QProcess::Running)
-      _imProcess->terminate();    
+      _imProcess->terminate();
   }
 
   /* Disconnect all of the TorControl object's signals */
@@ -419,7 +427,7 @@ MainWindow::close()
 
 /** Create and bind actions to events. Setup for initial
  * tray menu configuration. */
-void 
+void
 MainWindow::createActions()
 {
   _actionStartStopTor = new QAction(tr("Start Tor"), this);
@@ -429,7 +437,7 @@ MainWindow::createActions()
   connect(_actionExit, SIGNAL(triggered()), this, SLOT(close()));
 
   _actionShowBandwidth = new QAction(tr("Bandwidth Graph"), this);
-  connect(_actionShowBandwidth, SIGNAL(triggered()), 
+  connect(_actionShowBandwidth, SIGNAL(triggered()),
           _bandwidthGraph, SLOT(showWindow()));
   connect(ui.lblBandwidthGraph, SIGNAL(clicked()),
           _bandwidthGraph, SLOT(showWindow()));
@@ -439,9 +447,13 @@ MainWindow::createActions()
           _messageLog, SLOT(showWindow()));
   connect(ui.lblMessageLog, SIGNAL(clicked()),
           _messageLog, SLOT(showWindow()));
+  connect(ui.lblMessageLog, SIGNAL(clicked()),
+          _warnTimer, SLOT(stop()));
+  connect(ui.lblMessageLog, SIGNAL(clicked()),
+          ui.lblMessageLog, SLOT(disableFlashing()));
 
   _actionShowNetworkMap = new QAction(tr("Network Map"), this);
-  connect(_actionShowNetworkMap, SIGNAL(triggered()), 
+  connect(_actionShowNetworkMap, SIGNAL(triggered()),
           _netViewer, SLOT(showWindow()));
   connect(ui.lblViewNetwork, SIGNAL(clicked()),
           _netViewer, SLOT(showWindow()));
@@ -451,7 +463,7 @@ MainWindow::createActions()
 
   _actionShowConfig = new QAction(tr("Settings"), this);
   connect(_actionShowConfig, SIGNAL(triggered()), this, SLOT(showConfigDialog()));
-  
+
   _actionShowAbout = new QAction(tr("About"), this);
   connect(_actionShowAbout, SIGNAL(triggered()), this, SLOT(showAboutDialog()));
 
@@ -499,9 +511,9 @@ MainWindow::createTrayIcon()
 #endif
 }
 
-/** Creates a QMenu object that contains QActions which compose the system 
+/** Creates a QMenu object that contains QActions which compose the system
  * tray menu. */
-QMenu* 
+QMenu*
 MainWindow::createTrayMenu()
 {
   QMenu *menu = new QMenu(this);
@@ -513,7 +525,7 @@ MainWindow::createTrayMenu()
   menu->addAction(_actionNewIdentity);
   menu->addSeparator();
   menu->addAction(_actionShowControlPanel);
-  
+
 #if !defined(Q_WS_MAC)
   /* These aren't added to the dock menu on Mac, since they are in the
    * standard Mac locations in the menu bar. */
@@ -573,7 +585,7 @@ MainWindow::createMenuBar()
   viewMenu->addAction(_actionShowBandwidth);
   viewMenu->addAction(_actionShowMessageLog);
   viewMenu->addAction(_actionShowNetworkMap);
-  
+
   QMenu *helpMenu = _menuBar->addMenu(tr("Help"));
   _actionShowHelp->setText(tr("Vidalia Help"));
   helpMenu->addAction(_actionShowHelp);
@@ -612,31 +624,43 @@ MainWindow::launchBrowserFromDirectory()
 {
   VidaliaSettings settings;
 
+  /** Directory for the browser */
   QString browserDirectory = settings.getBrowserDirectory();
-  QString browserDirectoryFilename = settings.getBrowserExecutable();
+  if(QDir(browserDirectory).isRelative())
+    browserDirectory = QDir(QDir::toNativeSeparators(QCoreApplication::applicationDirPath()
+                                                     + "/" + browserDirectory)).canonicalPath();
+
+  /** Relative path to the browser executable from the browserDirectory */
+  QString browserExecutable = QDir::toNativeSeparators(browserDirectory + "/" + settings.getBrowserExecutable());
+
+  /** Relative path to profile from the browserDirectory */
+  QString profileDirectory = QDir::toNativeSeparators(settings.getProfileDirectory());
+  /** Default profile to copy from */
+  QString defaultProfileDirectory = QDir::toNativeSeparators(settings.getDefaultProfileDirectory());
+
+  /** Relative path to the plugins directory from the browserDirectory */
+  QString pluginsDirectory = QDir::toNativeSeparators(settings.getPluginsDirectory());
+  /** Relative path to the default plugins directory from the browserDirectory */
+  QString defaultPluginsDirectory = QDir::toNativeSeparators(settings.getDefaultPluginsDirectory());
+
+  QString profileDir = QDir(browserDirectory + "/" + profileDirectory).canonicalPath();
 
   _browserProcess->setEnvironment(updateBrowserEnv());
 
-  /* The browser is in <browserDirectory>/App/Firefox/<browserDirectoryFilename> */
-  QString browserExecutable =
-    QDir::toNativeSeparators(browserDirectory + "/App/Firefox/" + browserDirectoryFilename);
-  /* The profile is in <browserDirectory>/Data/profile */
-  QString profileDir =
-    QDir::toNativeSeparators(browserDirectory + "/Data/profile");
-
-  /* Copy the profile directory if it's not already there */
   QDir browserDirObj = QDir(browserDirectory);
 
   /* Copy the profile directory if it's not already there */
-  if (!browserDirObj.exists("Data/profile")) {
-    browserDirObj.mkdir("Data/profile");
-    copy_dir(browserDirectory + "/App/DefaultData/profile", browserDirectory + "/Data/profile");
+  if (!browserDirObj.exists(profileDirectory)) {
+    browserDirObj.mkdir(profileDirectory);
+    copy_dir(QDir(browserDirectory + "/" + defaultProfileDirectory).canonicalPath(),
+             QDir(browserDirectory + "/" + profileDirectory).canonicalPath());
   }
 
-  /* Copy the plugins directory if it's not already there */
-  if (!browserDirObj.exists("Data/plugins")) {
-    browserDirObj.mkdir("Data/plugins");
-    copy_dir(browserDirectory + "/App/DefaultData/plugins", browserDirectory + "/Data/plugins");
+  /* Copy the pluginss directory if it's not already there */
+  if (!browserDirObj.exists(pluginsDirectory)) {
+    browserDirObj.mkdir(pluginsDirectory);
+    copy_dir(QDir(browserDirectory + "/" + defaultPluginsDirectory).canonicalPath(),
+             QDir(browserDirectory + "/" + pluginsDirectory).canonicalPath());
   }
 
   /* Build the command line arguments */
@@ -647,7 +671,8 @@ MainWindow::launchBrowserFromDirectory()
   commandLine << profileDir;
 
   /* Launch the browser */
-  _browserProcess->start(browserExecutable, commandLine);
+  if(!_browserProcess->state() != QProcess::NotRunning)
+    _browserProcess->start(browserExecutable, commandLine);
   _browserProcess->toForeground();
 }
 
@@ -665,7 +690,8 @@ MainWindow::startSubprocesses()
   } else if (!(subprocess = settings.getBrowserExecutable()).isEmpty()) {
     /* BrowserDirectory is not set, but BrowserExecutable is; use this */
     _browserProcess->setEnvironment(updateBrowserEnv());
-    _browserProcess->start(subprocess, QStringList());
+    if(!_browserProcess->state() != QProcess::NotRunning)
+      _browserProcess->start(subprocess, QStringList());
     _browserProcess->toForeground();
   }
 
@@ -741,7 +767,7 @@ MainWindow::onCheckForBrowser()
 
   /* The browser isn't running, exit Vidalia */
   vApp->quit();
-#endif  
+#endif
 }
 
 /** Called when the web browser failed to start, for example, because the path
@@ -750,7 +776,7 @@ void
 MainWindow::onBrowserFailed(QString errmsg)
 {
   Q_UNUSED(errmsg);
- 
+
   /* Display an error message and see if the user wants some help */
   VMessageBox::warning(this, tr("Error starting web browser"),
               tr("Vidalia was unable to start the configured web browser"),
@@ -763,7 +789,7 @@ void
 MainWindow::onIMFailed(QString errmsg)
 {
   Q_UNUSED(errmsg);
- 
+
   /* Display an error message and see if the user wants some help */
   VMessageBox::warning(this, tr("Error starting IM client"),
               tr("Vidalia was unable to start the configured IM client"),
@@ -785,7 +811,7 @@ void
 MainWindow::onProxyFailed(QString errmsg)
 {
   Q_UNUSED(errmsg);
- 
+
   /* Display an error message and see if the user wants some help */
   VMessageBox::warning(this, tr("Error starting proxy server"),
               tr("Vidalia was unable to start the configured proxy server"),
@@ -798,7 +824,7 @@ void
 MainWindow::bootstrapStatusChanged(const BootstrapStatus &bs)
 {
   int percentComplete = STARTUP_PROGRESS_BOOTSTRAPPING + bs.percentComplete();
-  bool warn = (bs.severity() == tc::WarnSeverity && 
+  bool warn = (bs.severity() == tc::WarnSeverity &&
                bs.recommendedAction() != BootstrapStatus::RecommendIgnore);
 
   QString description;
@@ -837,7 +863,7 @@ MainWindow::bootstrapStatusChanged(const BootstrapStatus &bs)
       warn = false; /* probably false anyway */
       break;
     default:
-      description = tr("Unrecognized startup status");
+      description = bs.description();
   }
   if (warn) {
     QString reason;
@@ -883,7 +909,7 @@ MainWindow::updateTorStatus(TorStatus status)
   QString statusText, actionText;
   QString trayIconFile, statusIconFile;
   TorStatus prevStatus = _status;
- 
+
   vNotice("Tor status changed from '%1' to '%2'.")
     .arg(toString(prevStatus)).arg(toString(status));
   _status = status;
@@ -910,14 +936,14 @@ MainWindow::updateTorStatus(TorStatus status)
       setStartupProgressVisible(false);
   } else if (status == Stopping) {
       if (_delayedShutdownStarted) {
-        statusText = tr("Your relay is shutting down.\n" 
+        statusText = tr("Your relay is shutting down.\n"
                         "Click 'Stop' again to stop your relay now.");
       } else {
         statusText = tr("Tor is shutting down");
       }
       trayIconFile = IMG_TOR_STOPPING;
       statusIconFile = IMG_TOR_STOPPING_48;
-      
+
       ui.lblStartStopTor->setStatusTip(tr("Stop Tor Now"));
   } else if (status == Started) {
       actionText = tr("Stop Tor");
@@ -928,7 +954,7 @@ MainWindow::updateTorStatus(TorStatus status)
       ui.lblStartStopTor->setText(actionText);
       ui.lblStartStopTor->setPixmap(QPixmap(IMG_STOP_TOR_48));
       ui.lblStartStopTor->setStatusTip(actionText);
-            
+
       /* XXX: This might need to be smarter if we ever start connecting other
        * slots to these triggered() and clicked() signals. */
       QObject::disconnect(_actionStartStopTor, SIGNAL(triggered()), this, 0);
@@ -1009,7 +1035,7 @@ MainWindow::setStartupProgress(int progressValue,
 
 /** Attempts to start Tor. If Tor fails to start, then startFailed() will be
  * called with an error message containing the reason. */
-void 
+void
 MainWindow::start()
 {
   TorSettings settings;
@@ -1040,6 +1066,21 @@ MainWindow::start()
   }
 
   QString torrc = settings.getTorrc();
+  QFileInfo torrcInfo(torrc);
+
+  if(QDir(torrcInfo.filePath()).isRelative()) {
+    torrc = QCoreApplication::applicationDirPath() + "/" + torrc;
+
+    QFileInfo newTorrcInfo(torrc);
+    if(!newTorrcInfo.exists() and torrcInfo.exists()) {
+      torrc = QDir(QCoreApplication::applicationDirPath()).relativeFilePath(torrcInfo.absoluteFilePath());
+      vWarn("Automigrating configuration for Torrc:\nOld path: %1\nNew path: %2")
+        .arg(newTorrcInfo.filePath())
+        .arg(torrc);
+      settings.setTorrc(torrc);
+      torrc = QCoreApplication::applicationDirPath() + "/" + torrc;
+    }
+  }
 
   if(settings.bootstrap()) {
     QString boottorrc = settings.bootstrapFrom();
@@ -1061,15 +1102,35 @@ MainWindow::start()
 
   /* Specify Tor's data directory, if different from the default */
   QString dataDirectory = settings.getDataDirectory();
-  QString expDataDirectory = expand_filename(dataDirectory);
+  QFileInfo dataDirectoryInfo(dataDirectory);
+
+  if(not dataDirectory.isEmpty() and QDir(dataDirectory).isRelative()) {
+    dataDirectory = QCoreApplication::applicationDirPath() + "/" + dataDirectory;
+
+    QFileInfo newDataDirectoryInfo(dataDirectory);
+    if(!newDataDirectoryInfo.exists() and dataDirectoryInfo.exists()) {
+      dataDirectory = QDir(QCoreApplication::applicationDirPath()).relativeFilePath(dataDirectoryInfo.absoluteFilePath());
+      vWarn("Automigrating configuration for DataDirectory:\nOld path: %1\nNew path: %2")
+        .arg(newDataDirectoryInfo.absoluteFilePath())
+        .arg(dataDirectory);
+      settings.setDataDirectory(dataDirectory);
+      dataDirectory = QCoreApplication::applicationDirPath() + "/" + dataDirectory;
+    }
+  }
+
+  QString expDataDirectory = QDir(expand_filename(dataDirectory)).canonicalPath();
   if (!dataDirectory.isEmpty())
     args << "DataDirectory" << expDataDirectory;
-  
+
   if(settings.getControlMethod() == ControlMethod::Port) {
     if(settings.autoControlPort()) {
+      QString portconf = QString("%1/port.conf").arg(expDataDirectory);
+      if(!QFile::remove(portconf))
+        vWarn(QString("Unable to remove %1, may be it didn't existed.").arg(portconf));
+
       args << "ControlPort" << "auto";
       args << "SocksPort" << "auto";
-      args << "ControlPortWriteToFile" << QString("%1/port.conf").arg(expDataDirectory);
+      args << "ControlPortWriteToFile" << portconf;
     } else {
       /* Add the intended control port value */
       quint16 controlPort = settings.getControlPort();
@@ -1080,7 +1141,9 @@ MainWindow::start()
     QString path = settings.getSocketPath();
     args << "ControlSocket" << path;
   }
-  
+
+  args << "__OwningControllerProcess" << QString::number(QCoreApplication::applicationPid());
+
   /* Add the control port authentication arguments */
   switch (settings.getAuthenticationMethod()) {
     case TorSettings::PasswordAuth:
@@ -1109,7 +1172,24 @@ MainWindow::start()
    * start. */
   _isIntentionalExit = true;
   /* Kick off the Tor process */
-  _torControl->start(settings.getExecutable(), args);
+  QString torExecutable = settings.getExecutable();
+  QFileInfo torExecutableInfo(torExecutable);
+
+  if(QDir(torExecutableInfo.filePath()).isRelative()) {
+    torExecutable = QCoreApplication::applicationDirPath() + "/" + torExecutable;
+
+    QFileInfo newTorExecutableInfo(torExecutable);
+    if(!newTorExecutableInfo.exists() and torExecutableInfo.exists()) {
+      torExecutable = QDir(QCoreApplication::applicationDirPath()).relativeFilePath(torExecutableInfo.absoluteFilePath());
+      vWarn("Automigrating configuration for TorExecutable:\nOld path: %1\nNew path: %2")
+        .arg(newTorExecutableInfo.filePath())
+        .arg(torExecutable);
+      settings.setExecutable(torExecutable);
+      torExecutable = QCoreApplication::applicationDirPath() + "/" + torExecutable;
+    }
+  }
+
+  _torControl->start(torExecutable, args);
 }
 
 /** Called when the user changes a setting that needs Tor restarting */
@@ -1130,7 +1210,7 @@ MainWindow::startFailed(QString errmsg)
    * that Qt gives us in this instance is almost always "Unknown Error". That
    * will make users sad. */
   Q_UNUSED(errmsg);
- 
+
   updateTorStatus(Stopped);
 
   /* Display an error message and see if the user wants some help */
@@ -1154,7 +1234,7 @@ MainWindow::startFailed(QString errmsg)
 
 /** Slot: Called when the Tor process is started. It will connect the control
  * socket and set the icons and tooltips accordingly. */
-void 
+void
 MainWindow::started()
 {
   TorSettings settings;
@@ -1171,10 +1251,13 @@ MainWindow::started()
   /* Try to connect to Tor's control port */
   if(settings.autoControlPort()) {
     QString dataDirectory = settings.getDataDirectory();
-    QFile file(QString("%1/port.conf").arg(expand_filename(dataDirectory)));
+    if(QDir(dataDirectory).isRelative())
+      dataDirectory = QCoreApplication::applicationDirPath() + "/" + dataDirectory;
+
+    QFile file(QString("%1/port.conf").arg(QDir(expand_filename(dataDirectory)).canonicalPath()));
     int tries = 0, maxtries = 5;
     while((!file.open(QIODevice::ReadOnly | QIODevice::Text)) and
-	  (tries++ < maxtries)) {
+          (tries++ < maxtries)) {
       vWarn(QString("This is try number: %1.").arg(tries));
 #if defined(Q_WS_WIN)
       Sleep(1000);
@@ -1231,9 +1314,9 @@ void
 MainWindow::connectFailed(QString errmsg)
 {
   /* Ok, ok. It really isn't going to connect. I give up. */
-  int response = VMessageBox::warning(this, 
+  int response = VMessageBox::warning(this,
                    tr("Connection Error"), p(errmsg),
-                   VMessageBox::Ok|VMessageBox::Default|VMessageBox::Escape, 
+                   VMessageBox::Ok|VMessageBox::Default|VMessageBox::Escape,
                    VMessageBox::Retry, VMessageBox::Help);
 
 
@@ -1270,16 +1353,16 @@ MainWindow::stop()
                         "open connections from clients.\n\n"
                         "Would you like to shutdown gracefully and "
                         "give clients time to find a new relay?"),
-                        VMessageBox::Yes|VMessageBox::Default, 
-                        VMessageBox::No, 
+                        VMessageBox::Yes|VMessageBox::Default,
+                        VMessageBox::No,
                         VMessageBox::Cancel|VMessageBox::Escape);
     if (response == VMessageBox::Yes)
       _delayedShutdownStarted = true;
     else if (response == VMessageBox::Cancel)
       return false;
   }
-  
-  prevStatus = updateTorStatus(Stopping);  
+
+  prevStatus = updateTorStatus(Stopping);
   if (_delayedShutdownStarted) {
     /* Start a delayed shutdown */
     rc = _torControl->signal(TorSignal::Shutdown, &errmsg);
@@ -1288,15 +1371,15 @@ MainWindow::stop()
     _isIntentionalExit = true;
     rc = _torControl->stop(&errmsg);
   }
-  
+
   if (!rc) {
     /* We couldn't tell Tor to stop, for some reason. */
     int response = VMessageBox::warning(this, tr("Error Shutting Down"),
-                     p(tr("Vidalia was unable to stop the Tor software.")) 
+                     p(tr("Vidalia was unable to stop the Tor software."))
                        + p(errmsg),
-                     VMessageBox::Ok|VMessageBox::Default|VMessageBox::Escape, 
+                     VMessageBox::Ok|VMessageBox::Default|VMessageBox::Escape,
                      VMessageBox::Help);
-      
+
     if (response == VMessageBox::Help) {
       /* Show some troubleshooting help */
       showHelpDialog("troubleshooting.stop");
@@ -1311,7 +1394,7 @@ MainWindow::stop()
 
 /** Slot: Called when the Tor process has exited. It will adjust the tray
  * icons and tooltips accordingly. */
-void 
+void
 MainWindow::stopped(int exitCode, QProcess::ExitStatus exitStatus)
 {
   updateTorStatus(Stopped);
@@ -1328,11 +1411,11 @@ MainWindow::stopped(int exitCode, QProcess::ExitStatus exitStatus)
                      "unexpectedly.\n\n"
                      "Please check the message log for recent "
                      "warning or error messages."),
-                  VMessageBox::Ok|VMessageBox::Escape, 
+                  VMessageBox::Ok|VMessageBox::Escape,
                   VMessageBox::ShowLog|VMessageBox::Default,
                   VMessageBox::Help);
       if (ret == VMessageBox::ShowLog)
-        _messageLog->showWindow();  
+        _messageLog->showWindow();
       else if (ret == VMessageBox::Help)
         showHelpDialog("troubleshooting.torexited");
     }
@@ -1344,6 +1427,11 @@ void
 MainWindow::connected()
 {
   authenticate();
+  if(_torControl->isVidaliaRunningTor()) {
+    QString err;
+    if(!_torControl->takeOwnership(&err))
+      vWarn(err);
+  }
 }
 
 /** Called when Vidalia wants to disconnect from a Tor it did not start. */
@@ -1362,7 +1450,7 @@ MainWindow::disconnected()
      * connection as "Tor is stopped". */
     updateTorStatus(Stopped);
   }
-  
+
   /*XXX We should warn here if we get disconnected when we didn't intend to */
   _actionNewIdentity->setEnabled(false);
   ui.lblNewIdentity->setEnabled(false);
@@ -1378,12 +1466,12 @@ MainWindow::authenticate()
   TorSettings::AuthenticationMethod authMethod;
   TorSettings settings;
   ProtocolInfo pi;
-  
+
   updateTorStatus(Authenticating);
   setStartupProgress(STARTUP_PROGRESS_AUTHENTICATING,
                      tr("Authenticating to Tor"));
 
-  authMethod = settings.getAuthenticationMethod(); 
+  authMethod = settings.getAuthenticationMethod();
   pi = _torControl->protocolInfo();
   if (!pi.isEmpty()) {
     QStringList authMethods = pi.authMethods();
@@ -1394,7 +1482,7 @@ MainWindow::authenticate()
     else if (authMethods.contains("NULL"))
       authMethod = TorSettings::NullAuth;
   }
-  
+
   if (authMethod == TorSettings::CookieAuth) {
     /* Try to load an auth cookie and send it to Tor */
     QByteArray cookie = loadControlCookie(pi.cookieAuthFile());
@@ -1409,7 +1497,7 @@ MainWindow::authenticate()
                          "'control_auth_cookie' yourself?")),
                 VMessageBox::Browse|VMessageBox::Default,
                 VMessageBox::Cancel|VMessageBox::Escape);
-      
+
       if (ret == VMessageBox::Cancel)
         goto cancel;
       QString cookieDir = QFileDialog::getOpenFileName(this,
@@ -1421,6 +1509,11 @@ MainWindow::authenticate()
       cookieDir = QFileInfo(cookieDir).absolutePath();
       cookie = loadControlCookie(cookieDir);
     }
+    if(cookie.size() != 32) {
+      vWarn(QString("Cookie length has to be exactly 32 bytes long. Found %1 bytes")
+            .arg(cookie.size()));
+      goto cancel;
+    }
     vNotice("Authenticating using 'cookie' authentication.");
     return _torControl->authenticate(cookie);
   } else if (authMethod == TorSettings::PasswordAuth) {
@@ -1453,14 +1546,14 @@ MainWindow::authenticated()
   QString errmsg;
 
   updateTorStatus(Authenticated);
-  
+
   /* If Tor doesn't have bootstrapping events, then update the current
    * status string and bump the progress bar along a bit. */
   if (_torControl->getTorVersion() < 0x020101) {
     setStartupProgress(STARTUP_PROGRESS_CIRCUITBUILD,
                        tr("Connecting to the Tor network"));
   }
-  
+
   /* Let people click on their beloved "New Identity" button */
   _actionNewIdentity->setEnabled(true);
   ui.lblNewIdentity->setEnabled(true);
@@ -1500,7 +1593,7 @@ void
 MainWindow::authenticationFailed(QString errmsg)
 {
   bool retry = false;
-  
+
   vWarn("Authentication failed: %1").arg(errmsg);
 
   /* Parsing log messages is evil, but we're left with little option */
@@ -1551,21 +1644,21 @@ MainWindow::authenticationFailed(QString errmsg)
     }
   } else {
     /* Something else went wrong */
-    int ret = VMessageBox::warning(this, 
+    int ret = VMessageBox::warning(this,
                 tr("Authentication Error"),
                 p(tr("Vidalia was unable to authenticate to the Tor software. "
-                     "(%1)").arg(errmsg)) + 
+                     "(%1)").arg(errmsg)) +
                 p(tr("Please check your control port authentication "
                      "settings.")),
                 VMessageBox::ShowSettings|VMessageBox::Default,
                 VMessageBox::Cancel|VMessageBox::Escape);
-    
+
     if (ret == VMessageBox::ShowSettings)
       showConfigDialog(ConfigDialog::Advanced);
   }
-  
+
   if (_torControl->isRunning())
-    if (_isVidaliaRunningTor) 
+    if (_isVidaliaRunningTor)
       stop();
     else
       disconnect();
@@ -1592,14 +1685,14 @@ MainWindow::loadControlCookie(QString cookiePath)
     QString dataDir = settings.getDataDirectory();
     if (!dataDir.isEmpty())
       pathList << dataDir;
-      
+
 #if defined(Q_WS_WIN)
     pathList << expand_filename("%APPDATA%\\Tor");
 #else
     pathList << expand_filename("~/.tor");
 #endif
   }
-  
+
   /* Search for the cookie file */
   foreach (QString path, pathList) {
     QString cookieFile = QFileInfo(path).isFile() ?
@@ -1607,7 +1700,7 @@ MainWindow::loadControlCookie(QString cookiePath)
     vDebug("Checking for authentication cookie in '%1'").arg(cookieFile);
     if (!QFileInfo(cookieFile).exists())
       continue;
-    
+
     authCookie.setFileName(cookieFile);
     if (authCookie.open(QIODevice::ReadOnly)) {
       vInfo("Reading authentication cookie from '%1'").arg(cookieFile);
@@ -1648,6 +1741,9 @@ MainWindow::circuitEstablished()
 void
 MainWindow::checkTorVersion()
 {
+  VidaliaSettings settings;
+  if(settings.skipVersionCheck())
+    return;
   QString status;
   if (_torControl->getInfo("status/version/current", status)) {
     if (!status.compare("old", Qt::CaseInsensitive)
@@ -1734,7 +1830,7 @@ MainWindow::warnDangerousPort(quint16 port, bool rejected)
     case 109:
     case 110:
     case 143:
-      application = tr("(probably an email client)"); 
+      application = tr("(probably an email client)");
       break;
 
     default:
@@ -1846,7 +1942,7 @@ MainWindow::newIdentity()
   QString errmsg;
 
   /* Send the NEWNYM signal. If message balloons are supported and the NEWNYM
-   * is successful, we will show the result as a balloon. Otherwise, we'll 
+   * is successful, we will show the result as a balloon. Otherwise, we'll
    * just use a message box. */
   if (_torControl->signal(TorSignal::NewNym, &errmsg)) {
     /* NEWNYM signal was successful */
@@ -1858,7 +1954,7 @@ MainWindow::newIdentity()
     /* Disable the New Identity button for MIN_NEWIDENTITY_INTERVAL */
     _actionNewIdentity->setEnabled(false);
     ui.lblNewIdentity->setEnabled(false);
-    QTimer::singleShot(MIN_NEWIDENTITY_INTERVAL, 
+    QTimer::singleShot(MIN_NEWIDENTITY_INTERVAL,
                        this, SLOT(enableNewIdentity()));
 
     if (QSystemTrayIcon::supportsMessages())
@@ -1867,7 +1963,7 @@ MainWindow::newIdentity()
       VMessageBox::information(this, title, message, VMessageBox::Ok);
   } else {
     /* NEWNYM signal failed */
-    VMessageBox::warning(this, 
+    VMessageBox::warning(this,
       tr("Failed to Create New Identity"), errmsg, VMessageBox::Ok);
   }
 }
@@ -2053,7 +2149,7 @@ MainWindow::installUpdatesFailed(const QString &errmsg)
 
   VMessageBox::warning(this, tr("Installation Failed"),
                        p(tr("Vidalia was unable to install your software updates."))
-                         + p(tr("The following error occurred:")) 
+                         + p(tr("The following error occurred:"))
                          + p(errmsg),
                        VMessageBox::Ok);
 
@@ -2087,11 +2183,11 @@ MainWindow::updateBrowserEnv() {
     env << QString("TOR_SOCKS_PORT=%1").arg(port);
 
     vInfo(QString("Using automatic ControlPort and SocksPort configuration:\n"
-		  "   ControlPort=%1\n   SocksPort=%2\n   Host=%3\n   Configuration file:%4")
-	  .arg(_autoControlPort)
-	  .arg(port)
-	  .arg(addrPort.at(0))
-	  .arg(QString("%1/port.conf").arg(expand_filename(settings.getDataDirectory()))));
+                  "   ControlPort=%1\n   SocksPort=%2\n   Host=%3\n   Configuration file:%4")
+          .arg(_autoControlPort)
+          .arg(port)
+          .arg(addrPort.at(0))
+          .arg(QString("%1/port.conf").arg(expand_filename(settings.getDataDirectory()))));
   }
 
   if(settings.getAuthenticationMethod() == TorSettings::PasswordAuth) {
@@ -2101,3 +2197,22 @@ MainWindow::updateBrowserEnv() {
 
   return env;
 }
+
+void
+MainWindow::log(tc::Severity type, const QString &message)
+{
+  if(type == tc::WarnSeverity)
+    _warnTimer->start(1000);
+}
+
+void
+MainWindow::warnButton()
+{
+  if(_flashToggle) {
+    ui.lblMessageLog->enableFlashing();
+  } else {
+    ui.lblMessageLog->disableFlashing();
+  }
+
+  _flashToggle = !_flashToggle;
+}
